<?php namespace EwTools\Twig;

use Twig_Environment;
use Twig_Error_Runtime;
use Twig_Source;
use Twig_SourceContextLoaderInterface;

/**
 * Environment
 *
 * @link    http://infomaker.se
 * @package EwTools\Twig
 * @since   EwTools\Twig\Environment 0.1
 */
class Environment extends Twig_Environment {
    
    /**
     * Loads a specific template and compiles it. If cache is on, save the compiled class in cac3he to be evaluated
     * later on.
     *
     * @param string   $name
     * @param int|null $index
     *
     * @throws Twig_Error_Runtime
     * @since 0.1
     * @return \Twig_TemplateInterface
     */
    public function loadTemplate( $name, $index = null ) {
        $cls = $mainCls = $this->getTemplateClass( $name );
        
        if( isset( $this->loadedTemplates[ $cls ] ) ) {
            return $this->loadedTemplates[ $cls ];
        }
        
        if( ! class_exists( $cls, false ) ) {
            if( $this->cache instanceof objectCacheSystem ) {
                $key = $this->cache->generateKey( $name, $mainCls );
                
                // Fetch template from cache and load
                $this->cache->load( $key );
                
                if( ! class_exists( $cls, false ) ) {
                    $this->cache->write( $key, $this->getCompiledContent($name) );
                    $this->cache->load( $key );
                }
            }
            
            if( ! class_exists( $mainCls, false ) ) {
                eval( '?>' . $this->getCompiledContent($name) );
            }
            
            if( ! class_exists( $cls, false ) ) {
                throw new Twig_Error_Runtime( sprintf( 'Failed to load Twig template "%s", index "%s": cache is corrupted.', $name, $index ), -1, $source );
            }
        }
        
        return parent::loadTemplate( $name );
    }
    
    /**
     * Custom function for fetching and compiling the source code for the template
     *
     * @param $name
     *
     * @since 0.1
     * @return string
     */
    protected function getCompiledContent( $name ) {
        $loader = $this->getLoader();
    
        if( ! $loader instanceof Twig_SourceContextLoaderInterface ) {
            $source = new Twig_Source( $loader->getSource( $name ), $name );
        } else {
            $source = $loader->getSourceContext( $name );
        }
    
        return $this->compileSource( $source );
    }
}